''*************************************''
''*       Vassal Board.spin  v1.1     *''
''*  (C) 2008 Branmuffin Industries   *''
''*************************************''


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
 
  videopin         =  17   '17-19    (uses three sequential pins for video output)
  videoenablepin   =  16   '   16    (uses one pin to toggle the video connection on)
  A6276pin         =  20   '20-23    (uses four sequential pins for serial communication with LED Driver)   
  Darlingtonpin    =   8   ' 8-11    (uses four sequential pins for 5V Relay Control)
                           '12-15    (uses four sequential pins for 5V Relay Control)                         
  ADC0838_Apin     =   0   ' 0- 3    (uses three sequential pins for serial communication with ADC0838)   
  ADC0838_Bpin     =   4   ' 4- 7    (uses three sequential pins for serial communication with ADC0838)   
  KingSerialpin    =  30   '30-31   (uses two sequential pins for serial communication with King)          
 
  Off       = 0
  On        = 1
  
  LeftPack  = 0 
  RightPack = 1

  
VAR

     byte VassalPos
     byte DecivoltsA[4]
     byte DecivoltsB[4]

OBJ

  MyVideo          : "VassalVideo"       
  MyDarlington     : "VassalDarlington" 
  MySensors        : "VassalSensors"    

  
  'MyLEDArray       : "VassalLEDArray" 
  'MyHistoryA[4]       : "SingleBatteryHistory"

  
PUB Main 

   InitVassal   ' initializes the vassal and all its slave objects
   RunVassal  ' loops through the standard vassal board sequence

   repeat 
      'RunDemoLeftSide      ' simulates running the vassal  
      'RunDemoRightSide
      'RunDischargeMode   ' scans between the batteries to check voltage on each cell
      
PUB InitVassal

                                            'Initialize Video by sending its start pin         
   MyVideo.Init( videopin, videoenablepin ) '(uses three sequential pins and one to toggle video on/off) 

   
                                            'Initialize Darlington by sending its start pin
   MyDarlington.Init( Darlingtonpin )       '(uses four sequential pins for 5V Relay Control)


                                            'Initialize SensorsA by sending its start pin  
   MySensors.SensorInit( ADC0838_Apin )    '(uses three sequential pins for serial communication with ADC0838)

      
                                            'Initialize LEDArray by sending its start pin
   'MyLEDArray.Init( A6276pin )              '(uses four sequential pins for 5V Relay Control) 


PUB RunVassal
   repeat
      'RunGrossChargeMode
      'RunPolishChargeMode
      RunDischargeMode
    
PUB RunGrossChargeMode

PUB RunPolishChargeMode        

PUB RunDischargeMode

   
   VassalGetVoltage( LeftPack , 1 )
   waitcnt( 4_000_000 + cnt)
   VassalGetVoltage( LeftPack , 2 )  
   waitcnt( 4_000_000 + cnt)  
   VassalGetVoltage( LeftPack , 3 )
   waitcnt( 4_000_000 + cnt)
   VassalGetVoltage( LeftPack , 4 )
   waitcnt( 4_000_000 + cnt)    
   

PUB RunDemoLeftSide                                    ' simulates running the vassal    

   DecivoltsA[0] := 240
   DecivoltsA[1] := 46     
   DecivoltsA[2] := 128    
   DecivoltsA[3] := 22
   
   DecivoltsB[0] := 12
   DecivoltsB[1] := 16     
   DecivoltsB[2] := 240    
   DecivoltsB[3] := 170
   
   repeat VassalPos from 1 to 4
      
      'MyVideo.UpdateVoltage( LeftPack, VassalPos, DecivoltsA[VassalPos-1] )
      'MyVideo.UpdateVoltage( RightPack, VassalPos, DecivoltsB[VassalPos-1] )      
       
   repeat VassalPos from 1 to 4
     
       VassalChargeOff( LeftPack )          'turn charging off
       VassalScan( LeftPack ) 
       VassalCharge( LeftPack , VassalPos )             'begin charge sequence on this battery
       repeat DecivoltsA[VassalPos-1] from DecivoltsA[VassalPos-1] to 255
           DecivoltsA[VassalPos-1] := DecivoltsA[VassalPos-1] + 1
           'MyVideo.UpdateVoltage( LeftPack, VassalPos, DecivoltsA[VassalPos-1] )
           waitcnt( 900_000 + cnt)  
       VassalChargeComplete( LeftPack , VassalPos ) 
          
   VassalScan( LeftPack ) 
   VassalChargeOff( LeftPack )                    'turn charging off

PUB RunDemoRightSide  

   repeat VassalPos from 1 to 4
     
       VassalChargeOff( RightPack )                            'turn charging off
       VassalScan( RightPack ) 
       VassalCharge( RightPack , VassalPos )            'begin charge sequence on this battery
        repeat DecivoltsB[VassalPos-1] from DecivoltsB[VassalPos-1] to 255 step 1
           DecivoltsB[VassalPos-1] := DecivoltsB[VassalPos-1] + 1
           'MyVideo.UpdateVoltage( RightPack, VassalPos, DecivoltsB[VassalPos-1] )
           waitcnt( 900_000 + cnt)  
       VassalChargeComplete( RightPack , VassalPos ) 
          
   VassalScan( RightPack ) 
   VassalChargeOff( RightPack )                   'turn charging off
   
   VassalRecheckCharge( LeftPack )
   VassalRecheckCharge( RightPack )    
   
PRI VassalChargeOff ( _Pack )                  'discontinue charging on all cells for both packs

       'MyDarlington.ChargeOff( _Pack ) 
       'MyLEDArray.ChargeOff( _Pack )     
       MyVideo.UpdateChargeOff( _Pack )
       waitcnt( 100_000_000 + cnt)     'Wait for 1 seconds

        
PRI VassalCharge( _Pack , _Batt )   'begin charge sequence on this battery  

    'MyDarlington.Charge( _Pack , _Batt )           'tell the darlington to charge this cell             
    'MyLEDArray.Charging( _Pack , _Batt )
    MyVideo.UpdateCharging( _Pack, _Batt, 0 ) 'tells the video display that charging has started on this cell and the current  

    waitcnt( 100_000_000 + cnt)     'Wait for 5 seconds
  
   'tell the LED array to reflect that change

PRI VassalGetVoltage ( _Pack , _Batt ) | tempo 'get the voltage of this cell using the ADC and draw it to the screen           
   
    MyVideo.UpdateCharging( _Pack, _Batt, 1 )             'tells the video display that charging has started on this cell 
    MyDarlington.Select( _Pack , _Batt )                  'set the darlington to connect to the specified cell
    waitcnt( 4_000_000 + cnt)     'Wait for 2 seconds    
    repeat tempo from 0 to 200
         DecivoltsA[_Batt-1] := MySensors.GetConnectedBattVoltage    'use vassal sensors to update battery voltage
         MyVideo.UpdateVoltage( _Pack, _Batt, DecivoltsA[_Batt-1] )  'send that battery voltage measurement to the video screen to be drawn
                                       
    waitcnt( 100_000_000 + cnt)     'Wait for 5 seconds

    
                  
PRI VassalChargeComplete( _Pack , _Batt ) ' distributes the fact that this battery has finished charging

   'MyDarlington.ChargeOff( _Pack )      
   'MyLEDArray.ChargingComplete ( _Pack , _Batt ) 'tells the LED array that charging has finished on this cell
   'MyVideo.UpdateChargeComplete( _Pack, _Batt ) 'tells the video display that charging has finished on this cell

   
PRI VassalScan( _Pack )                  ' puts everything in scan mode for this pack

   'MyVideo.UpdateScanning( _Pack ) ' tells the video display that scanning has begun on this pack    
   'MyDarlington.ChargeOff( _Pack ) ' turns charging off
   'MyLEDArray.Scanning( _Pack )    ' illuminates scanning animation on LED Array
   'VassalGetADCData( _Pack)

PRI VassalRecheckCharge( _Pack )     'assumes nothing is charged and rechecks the charge on the cells

   'MyDarlington.ChargeOff( _Pack )
   'MyLEDArray.GreenOff( _Pack )
   'MyVideo.UpdateVoltage( _Pack, _Batt, _Voltage )
   
PRI VassalGetADCData( _Pack) | VassalTemp
{{

    'MySensors[Pack].GetADCData

    'MyVideo.UpdateCharging( _Pack, _Batt, current[_Pack] )  redraw current being pulled number on tv screen after reading current from ADC
    'MyVideo.UpdateVoltage( _Pack, _Batt, _Voltage )
    repeat VassalTemp from 0 to 3 step 1    
         If _Pack == LeftPack      
            DecivoltsA[VassalTemp] = mySensors.ADC         
         else       
            DecivoltsB[VassalTemp]       
        MyVideo.UpdateVoltage( _Pack, VassalTemp, _Voltage )
        
}}

PRI VassalChargeDemo( _Pack , _Batt )

    'MyVideo.UpdateVoltage( _Pack, _Batt, )    'sends a voltage as 0-255 byte    
PRI Crap '*********************************   
          
    'turn charging off
    'get the data fron the ADC0838 A and B
    
    'decide the state of charge and what to do about it

    'adjust the darlington outputs
    
    'adjust the LED outputs using a new cog

    'adjust the video output accordingly

    'send serial communication with the king board
    