''*************************************''
''*        VassalVideo.spin  v1.0     *''
''*  (C) 2008 Branmuffin Industries   *''
''*************************************''

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  _stack = ($3000 + $3000 + 100) >> 2   'accomodate display memory and stack

  x_tiles = 16
  y_tiles = 12

  paramcount = 14       
  bitmap_base = $2000
  display_base = $5000

  lines = 5
  thickness = 2

  Off = 0
  On  = 1

  LeftPack  = 0 
  RightPack = 1  
VAR


  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]

    byte converted[4]
    byte stored[4]
    byte startpin  
    
OBJ

  tv    : "tv"
  gr    : "graphics"    
  Num   : "Numbers"   
 
PUB Init( _videostartpin, videoenablepin ) | i, kk, dx, dy, pp, pq, rr, numx, numchr

   'startpin := 17
   startpin := _videostartpin 
  ' Initialize Numbers   
   Num.Init                                 

  'start tv
  tvparams_pins := (startpin & $38) << 1 | (startpin & 4 == 4) & %0101    'uses specified pin to start video 
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from 0 to 63
    colors[i] := $00001010 * (i+4) & $F + $2B060C02

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 128, 96, bitmap_base)

    gr.clear                ' clear bitmap   
    gr.colorwidth(2,0)      ' thin and white
    gr.copy(display_base)   ' added this to try to solve the problems

    PrepBattMonitor
  
PUB PrepBattMonitor  | videotempforinit, videoxposforinit

    gr.colorwidth(2,0)
     gr.textmode(1, 1, 6, %1001)  
    
    gr.text(     -100,  60,    String("4.6v")   )        
    gr.text(     -100,  30,    String("4.2v")   )      
    gr.text(     -100,   0,    String("3.8v")   )
    gr.text(     -100, -30,    String("3.4v")   )      
    gr.text(     -100, -60,    String("3.0v")   )
    gr.text(     -100, -90,    String("2.6v")   )  
    'gr.text(      -90,  60,    String("Charging Battery A1")   )   

    videotempforinit := 0 
    gr.text(  -80, -90,    String("A")   )   
    gr.text(   35, -90,    String("B")   )
    
    repeat videoxposforinit from -70 to 0 step 20
       gr.colorwidth(2,0)
       gr.textmode(1, 1, 6, %0001 ) 
       videotempforinit := videotempforinit + 1
       gr.box(   videoxposforinit, -100, 15, 130 )
       gr.colorwidth(0,0) 
       gr.text(  videoxposforinit-1, -90,    Num.ToStr( videotempforinit,  Num#DDEC )   )
       gr.colorwidth(3,2) 
       gr.textmode(2, 2, 6, %0001 ) 
       'gr.text(  videoxposforinit+3, 45,    String("+")   )   
       
    videotempforinit := 0      
    gr.colorwidth(2,0)
    gr.textmode(1, 1, 6, %1001)  

    repeat videoxposforinit from 40 to 110 step 20    
       gr.colorwidth(2,0)
       gr.textmode(1, 1, 6, %0001 ) 
       videotempforinit := videotempforinit + 1   
       gr.box(videoxposforinit,-100, 15, 130 )
       gr.colorwidth(0,0)    
       gr.text(  videoxposforinit-1, -90,    Num.ToStr( videotempforinit,  Num#DDEC )   )
       gr.colorwidth(3,2) 
       gr.textmode(2, 2, 6, %0001 ) 
       'gr.text(  videoxposforinit+3, 45,    String("?")   )

    'copy bitmap to display
    gr.copy(display_base)
       
    gr.colorwidth(1,0)   ' make pixel sprite bigger       
    gr.textmode(1,1,6,5)  ' return text to proper font
    

PUB DrawDynamics 
    

PUB UpdateVoltage( _Pack, _Batt, _Voltage ) | videoxposforupdatevoltage 'takes a battery voltage and redraws it to the screen

    videoxposforupdatevoltage := 110*_Pack + 20*_Batt - 90      
    gr.colorwidth(2,0)
    gr.box ( videoxposforupdatevoltage , -100, 15, 130 ) 
    gr.colorwidth(1,0)
    gr.box ( videoxposforupdatevoltage , -100, 15, _Voltage / 2)
    gr.colorwidth(2,0) 
    gr.textmode(1, 1, 6, %0001 )   
    gr.text( videoxposforupdatevoltage -1, -90,    Num.ToStr( _Batt,  Num#DDEC )   )

    gr.colorwidth(0,0)
    gr.box (  -80, 60, 90, 20 )
    gr.colorwidth(2,0)     
    gr.text(  -30, 70,    String("Volts")   )    
    gr.text(  -60, 70,   Num.ToStr( _Voltage,  Num#DDEC )  )   
    gr.copy(display_base)   
   
PUB UpdateScanning( _Pack ) | videoxposforscanning

    videoxposforscanning := -80 + 110 * _Pack
    gr.colorwidth(0,0)
    gr.box ( videoxposforscanning-10, 80, 110, 20 )
        
    gr.textmode(1, 1, 6, %0001 )            
    gr.colorwidth(2,0)

    'If _Pack == LeftPack
    '     gr.text(   videoxposforscanning, 90,    String("Scanning Pack A")   )
    'else     
    '     gr.text(   videoxposforscanning, 90,    String("Scanning Pack B")   )
    gr.copy(display_base)

PUB UpdateChargeOff ( _Pack ) |  videoxposforchargeoff  
    videoxposforchargeoff := 110*_Pack - 80
    gr.colorwidth(0,0)
    gr.box ( videoxposforchargeoff , 35, 90, 70 )
    gr.copy(display_base) 
    
PUB UpdateChargeComplete( _Pack, _Batt ) |  videoxposforchargecomplete

    videoxposforchargecomplete := 110*_Pack + 20*_Batt - 90
    gr.colorwidth(0,0)
    gr.box ( videoxposforchargecomplete , 35, 15, 20 )
    gr.colorwidth(3,2) 
    gr.textmode(2, 2, 6, %0001 ) 
    gr.text(  videoxposforchargecomplete +3, 45,    String("*")   )   
    gr.copy(display_base)
    
PUB UpdateCharging( _Pack, _Batt, _Current ) | videotempforcharging, videoxposforcharging

    videotempforcharging := _Batt
    videoxposforcharging := 110*_Pack + 20*_Batt - 90      
    gr.colorwidth(0,0)
    gr.box ( videoxposforcharging , 35, 15, 20 )
    gr.colorwidth(1,2) 
    gr.textmode(2, 2, 6, %0001 ) 
    gr.text(  videoxposforcharging+3, 45,    String("+")   )   

    videoxposforcharging := 110 * _Pack - 70
    gr.colorwidth(0,0)
    gr.box ( videoxposforcharging -10, 80, 90, 20 )
    gr.box (  -80, 60, 90, 20 )   
    gr.textmode(1, 1, 6, %0001 )            
    gr.colorwidth(2,0)
    gr.text(  videoxposforcharging    , 90,    String("Charging")   )
    
    If _Pack == LeftPack
         gr.text(   videoxposforcharging +60, 90,    String("A")   )
    else     
         gr.text(   videoxposforcharging +60, 90,    String("B")   )
         
    gr.text(  videoxposforcharging+64 , 90,    Num.ToStr( videotempforcharging,  Num#DDEC )     )

    gr.text(  videoxposforcharging+10, 70,    String("3.83 Amps")   ) 

    gr.copy(display_base)
     
DAT

tvparams                long    0               'status
                        long    1               'enable
tvparams_pins           long    %001_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog
